@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2010-2011 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2010-2015 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2011 Hans de Goede <hdegoede@redhat.com>
  * Copyright (C) 2011 Debarshi Ray <debarshir@src.gnome.org>
  *
@@ -38,6 +38,7 @@
 #include "gusb-util.h"
 #include "gusb-device.h"
 #include "gusb-device-private.h"
+#include "gusb-interface-private.h"
 
 /**
  * GUsbDevicePrivate:
@@ -331,19 +332,26 @@ g_usb_device_libusb_error_to_gerror (GUsbDevice  *device,
 	case LIBUSB_ERROR_NO_MEM:
 	case LIBUSB_ERROR_OTHER:
 	case LIBUSB_ERROR_INTERRUPTED:
-		error_code = G_USB_DEVICE_ERROR_INTERNAL; break;
+		error_code = G_USB_DEVICE_ERROR_INTERNAL;
+		break;
 	case LIBUSB_ERROR_IO:
 	case LIBUSB_ERROR_OVERFLOW:
 	case LIBUSB_ERROR_PIPE:
-		error_code = G_USB_DEVICE_ERROR_IO; break;
+		error_code = G_USB_DEVICE_ERROR_IO;
+		break;
 	case LIBUSB_ERROR_TIMEOUT:
-		error_code = G_USB_DEVICE_ERROR_TIMED_OUT; break;
+		error_code = G_USB_DEVICE_ERROR_TIMED_OUT;
+		break;
 	case LIBUSB_ERROR_NOT_SUPPORTED:
-		error_code = G_USB_DEVICE_ERROR_NOT_SUPPORTED; break;
-	case LIBUSB_ERROR_NO_DEVICE:
+		error_code = G_USB_DEVICE_ERROR_NOT_SUPPORTED;
+		break;
 	case LIBUSB_ERROR_ACCESS:
+		error_code = G_USB_DEVICE_ERROR_PERMISSION_DENIED;
+		break;
+	case LIBUSB_ERROR_NO_DEVICE:
 	case LIBUSB_ERROR_BUSY:
-		error_code = G_USB_DEVICE_ERROR_NO_DEVICE; break;
+		error_code = G_USB_DEVICE_ERROR_NO_DEVICE;
+		break;
 	}
 
 	g_set_error (error, G_USB_DEVICE_ERROR, error_code,
@@ -477,6 +485,113 @@ g_usb_device_get_custom_index (GUsbDevice *device,
 	return idx;
 }
 
+/**
+ * g_usb_device_get_interface:
+ * @device: a #GUsbDevice
+ * @class_id: a device class, e.g. 0xff for VENDOR
+ * @subclass_id: a device subclass
+ * @protocol_id: a protocol number
+ * @error: a #GError, or %NULL
+ *
+ * Gets the first interface that matches the vendor class interface descriptor.
+ * If you want to find all the interfaces that match (there may be other
+ * 'alternate' interfaces you have to use g_usb_device_get_interfaces() and
+ * check each one manally.
+ *
+ * Return value: (transfer full): a #GUsbInterface or %NULL for not found
+ *
+ * Since: 0.2.8
+ **/
+GUsbInterface *
+g_usb_device_get_interface (GUsbDevice *device,
+			    guint8      class_id,
+			    guint8      subclass_id,
+			    guint8      protocol_id,
+			    GError    **error)
+{
+	const struct libusb_interface_descriptor *ifp;
+	gint rc;
+	GUsbInterface *interface = NULL;
+	guint i;
+	struct libusb_config_descriptor *config;
+
+	g_return_val_if_fail (G_USB_IS_DEVICE (device), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	rc = libusb_get_active_config_descriptor (device->priv->device, &config);
+	if (!g_usb_device_libusb_error_to_gerror (device, rc, error))
+		return NULL;
+
+	/* find the right data */
+	for (i = 0; i < config->bNumInterfaces; i++) {
+		ifp = &config->interface[i].altsetting[0];
+		if (ifp->bInterfaceClass != class_id)
+			continue;
+		if (ifp->bInterfaceSubClass != subclass_id)
+			continue;
+		if (ifp->bInterfaceProtocol != protocol_id)
+			continue;
+		interface = _g_usb_interface_new (ifp);
+		break;
+	}
+
+	/* nothing matched */
+	if (interface == NULL) {
+		g_set_error (error,
+			     G_USB_DEVICE_ERROR,
+			     G_USB_DEVICE_ERROR_NOT_SUPPORTED,
+			     "no interface for class 0x%02x, "
+			     "subclass 0x%02x and protocol 0x%02x",
+			     class_id, subclass_id, protocol_id);
+	}
+
+	libusb_free_config_descriptor (config);
+	return interface;
+}
+
+/**
+ * g_usb_device_get_interfaces:
+ * @device: a #GUsbDevice
+ * @error: a #GError, or %NULL
+ *
+ * Gets all the interfaces exported by the device.
+ *
+ * Return value: (transfer container) (element-type GUsbInterface): an array of interfaces or %NULL for error
+ *
+ * Since: 0.2.8
+ **/
+GPtrArray *
+g_usb_device_get_interfaces (GUsbDevice *device, GError **error)
+{
+	const struct libusb_interface_descriptor *ifp;
+	gint rc;
+	guint i;
+	guint j;
+	struct libusb_config_descriptor *config;
+	GPtrArray *array = NULL;
+
+	g_return_val_if_fail (G_USB_IS_DEVICE (device), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	rc = libusb_get_active_config_descriptor (device->priv->device, &config);
+	if (!g_usb_device_libusb_error_to_gerror (device, rc, error))
+		return NULL;
+
+	/* get all interfaces */
+	array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	for (i = 0; i < config->bNumInterfaces; i++) {
+		GUsbInterface *interface = NULL;
+		for (j = 0; j < (guint) config->interface[i].num_altsetting; j++) {
+			ifp = &config->interface[i].altsetting[j];
+			interface = _g_usb_interface_new (ifp);
+			g_ptr_array_add (array, interface);
+		}
+	}
+
+	libusb_free_config_descriptor (config);
+	return array;
+}
+
 /**
  * g_usb_device_close:
  * @device: a #GUsbDevice
@@ -528,6 +643,8 @@ g_usb_device_reset (GUsbDevice  *device,
 		return g_usb_device_not_open_error (device, error);
 
 	rc = libusb_reset_device (device->priv->handle);
+	if (rc == LIBUSB_ERROR_NOT_FOUND)
+		return TRUE;
 	return g_usb_device_libusb_error_to_gerror (device, rc, error);
 }
 
@@ -699,6 +816,38 @@ g_usb_device_release_interface (GUsbDevice                     *device,
 	return TRUE;
 }
 
+/**
+ * g_usb_device_set_interface_alt:
+ * @device: a #GUsbDevice
+ * @interface: bInterfaceNumber of the interface you wish to release
+ * @alt: alternative setting number
+ * @error: a #GError, or %NULL
+ *
+ * Sets an alternate setting on an interface.
+ *
+ * Return value: %TRUE on success
+ *
+ * Since: 0.2.8
+ **/
+gboolean
+g_usb_device_set_interface_alt (GUsbDevice *device, gint interface,
+                                guint8 alt, GError **error)
+{
+	gint rc;
+
+	g_return_val_if_fail (G_USB_IS_DEVICE (device), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	if (device->priv->handle == NULL)
+		return g_usb_device_not_open_error (device, error);
+
+	rc = libusb_set_interface_alt_setting (device->priv->handle, interface, (gint) alt);
+	if (rc != LIBUSB_SUCCESS)
+		return g_usb_device_libusb_error_to_gerror (device, rc, error);
+
+	return TRUE;
+}
+
 /**
  * g_usb_device_get_string_descriptor:
  * @desc_index: the index for the string descriptor to retreive
@@ -1549,6 +1698,24 @@ g_usb_device_get_pid (GUsbDevice *device)
 	return device->priv->desc.idProduct;
 }
 
+/**
+ * g_usb_device_get_release:
+ * @device: a #GUsbDevice
+ *
+ * Gets the BCD firmware version number for the device.
+ *
+ * Return value: a version number in BCD format.
+ *
+ * Since: 0.2.8
+ **/
+guint16
+g_usb_device_get_release (GUsbDevice *device)
+{
+	g_return_val_if_fail (G_USB_IS_DEVICE (device), 0);
+
+	return device->priv->desc.bcdDevice;
+}
+
 /**
  * g_usb_device_get_vid_as_str:
  * @device: a #GUsbDevice
